#!/usr/bin/env python
"""extract exif data

RCSid:
    $Id: exif.py,v 1.14 2006/04/02 19:45:21 sjg Exp $
"""

import getopt, os, sys
sys.path.append(os.path.dirname(sys.path[0]))
from contrib.exif import EXIF

def getKey(dict, key, default=None, raw=False):
    if not dict.has_key(key):
        return default
    if raw:
        return dict[Key].values
    else:
        return dict[key].printable

def floatVal(v):
    try:
        n,d = v.split('/')
        v = float(n) / float(d)
    except: pass
    return v

def reduceVal(v):
    if not v:
        return ''
    try:
        n,d = v.split('/')
        if int(n) > 1:
            d = int(d)/int(n)
            v = '1/%d' % d
    except: pass
    return v

def ecVal(ec):
    if not ec or ec == '0':
        return ''
    if ec.find('/') > 0:
        if ec[0] == '-':
            ec = ', EC %s' % ec
        else:
            ec = ', EC +%s' % ec
    else:
        v = int(ec)
        ec = ', EC %+d' % v
    return ec

def onelineExif(data):
    make = getKey(data, 'Image Make')
    model = getKey(data, 'Image Model')
    x = len(make)
    if len(model) < x or model[:x] != make:
        w = make.split()
        model = '%s %s' % (w[0], model)
    focal = floatVal(getKey(data, 'EXIF FocalLength'))
    Fu = float(floatVal(getKey(data, 'MakerNote FocalUnitsPerMM', 1)))
    minF = getKey(data, 'MakerNote ShortFocalLengthOfLensInFocalUnits', focal)
    maxF = getKey(data, 'MakerNote LongFocalLengthOfLensInFocalUnits', focal)
    if minF != maxF:
        if Fu > 1.0:
            lens = '%.2f-%.2f @ %.2fmm' % (float(minF) / Fu,
                                 float(maxF) / Fu, float(focal))
        else:
            lens = '%s-%s @ %smm' % (minF, maxF, focal)
    else:
        lens = '%smm' % focal
    av = floatVal(getKey(data, 'EXIF FNumber'))
    tv = reduceVal(getKey(data, 'EXIF ExposureTime'))
    ec = ecVal(getKey(data, 'EXIF ExposureBiasValue'))
    fec = getKey(data, 'MakerNote FlashBias', '0 EV')
    if fec == '0 EV':
        fec = ''
    else:
        fec = ', FEC %s' % fec
    iso = getKey(data, 'EXIF ISOSpeedRatings')
    if not iso:
        iso = getKey(data, 'MakerNote ISO')
    flash = getKey(data, 'EXIF Flash')
    if flash == 'Off' or flash == 'No':
        flash = ''
    else:
        # skip the 1st word ('Fired'), it isn't very interesting
        w = flash.split()
        if len(w) > 1:
            flash = ', Flash %s' % ' '.join(w[1:])
        else:
            flash = ', Flash'
    wb = getKey(data, 'MakerNote WhiteBalance', 'Auto')
    if wb == 'Auto':
        wb = ''
    else:
        wb = ', WB %s' % wb

    return '%s, %s, %ss f/%s ISO %s%s%s%s%s' % (model, lens, tv, av, iso, ec, fec, flash, wb)

# library test/debug function (dump given files)
if __name__ == '__main__':
    tags=[
        'EXIF DateTimeOriginal',
        'Image Model',
        'MakerNote FocalUnitsPerMM',
        'MakerNote ShortFocalLengthOfLensInFocalUnits',
        'MakerNote LongFocalLengthOfLensInFocalUnits',
        'EXIF FocalLength',
        'EXIF FNumber',
        'EXIF ExposureTime',
        'EXIF ISOSpeedRatings',
        'EXIF ExposureBiasValue',
        'MakerNote FlashBias',
        'EXIF Flash',
        'MakerNote WhiteBalance',
        ]
    html = False
    oneline = False
    verbose = False
    ext = None
    rotation = None
    
    try:
        opts, args = getopt.getopt(sys.argv[1:], "1hr:vt:e:")
    except getopt.GetoptError:
        print >> sys.stderr, "Use the source Luke!"
        sys.exit(2)
    for o, a in opts:
        if o == '-1':
            oneline = True
        elif o == '-e':
            ext = a
        elif o == '-h':
            html = True
        elif o == '-r':
            if a == '-':
                rotation = sys.stdout
            else:
                rotation = open(a, "a")
        elif o == '-t':
            if a == 'all' or a == '-':
                tags = []
            else:
                for t in a.split(','):
                    tags.append(t)
        elif o == '-v':
            verbose = True
            
    if len(args) < 1:
        print 'Usage: %s files...\n' % sys.argv[0]
        sys.exit(0)
        
    for filename in args:
        try:
            file=open(filename, 'rb')
        except:
            print filename, 'unreadable'
            print
            continue
        if not ext and ((not html and not oneline) or verbose):
            print filename+':'
        try:
            # data=process_file(file, 1) # with debug info
            data=EXIF.process_file(file)
            if not data:
                print 'No EXIF information found'
                continue
        except ValueError, e:
            print >> sys.stderr, filename + ": cannot decode EXIF: ", e
            continue

        if rotation:
            r = getKey(data, 'Image Orientation')
            #print '%s:%s' % (filename, r)
            if r and r.startswith('Rotated'):
                x = r.split(' ')
                v = int(x[-2])
                if x[-1] == 'CCW':
                    v = -v
                print >> rotation, '%s:%d' % (filename, v)
            
        if ext:
            out, junk = os.path.splitext(filename)
            out += ext
            sys.stdout = open(out, 'w')
            
        if oneline:
            if ext:
                print onelineExif(data)
            else:
                print '%s: %s' % (filename, onelineExif(data))
            continue
        
        if html:
            print '<table><tbody>'

        if len(tags) > 0:
            x = tags
        else:
            x = data.keys()
            x.sort()

        for i in x:
            if not data.has_key(i):
                for n in ['EXIF', 'MakerNote']:
                    t = '%s %s' % (n, i)
                    if data.has_key(t):
                        i = t
                        break
            if not data.has_key(i):
                continue
            if i in ('JPEGThumbnail', 'TIFFThumbnail'):
                continue
            try:
                ftype =  EXIF.FIELD_TYPES[data[i].field_type][2]
                value = data[i].printable
                if ftype == 'Ratio':
                    try:
                        if i == 'EXIF FNumber':
                            n,d = data[i].printable.split('/')
                            value = float(n) / float(d)
                    except: pass
                if html:
                    print '<tr><td>%s:</td><td>%s</td></tr>' % (i, value)
                elif verbose:
                    print '   %s (%s): %s' % (i, ftype, value)
                else:
                    print '%s: %s' % (i, value)
            except Exception, e:
                print 'error', i, '"', data[i], '"; ', e
        if html:
            print '</tbody></table>'
        if verbose and data.has_key('JPEGThumbnail'):
            print 'File has JPEG thumbnail'
        print
