:
# NAME:
#	hooks.sh - provide hooks for customization
#
# SYNOPSIS:
#	add_hooks HOOKS cmd [...]
#	run_hooks HOOKS [LIFO]
#	run_hooks_all HOOKS [LIFO]
#
# DESCRIPTION:
#	add_hooks simple adds the "cmd"s to the list "HOOKS".
#	run_hooks runs each "cmd" in $HOOKS and stops if any of them
#	return a bad status.
#	run_hooks_all does the same but does not stop on error.
#	If run_hooks or run_hooks_all is given a 2nd argument of LIFO
#	the hooks are run in the reverse order of calls to add_hooks.
#	

# RCSid:
#	$Id: hooks.sh,v 1.5 2006/03/20 20:34:03 sjg Exp $
#
#	@(#)Copyright (c) 2000 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 

# avoid multiple inclusion
_HOOKS_SH=:

add_hooks() {
    __h=$1; shift
    eval "$__h=\"\$$__h $*\""
}

__get_hooks() {
    eval "__h=\$$1"
    case "$2" in
    LIFO) __h2=
	for e in $__h
	do
	    __h2="$e $__h2"
	done
	__h="$__h2"
	;;
    esac
}	 
	    
run_hooks() {
    __get_hooks "$@"
    for e in $__h
    do
	$e || break
    done
}

run_hooks_all() {
    __get_hooks "$@"
    for e in $__h
    do
	$e
    done
}

case /$0 in
*/hooks-rev*) add_hooks HOOKS $*; run_hooks HOOKS LIFO;;
*/hooks*) add_hooks HOOKS $*; run_hooks HOOKS;;
esac
